/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana;

import java.util.Collection;

import de.bsvrz.iav.fuzzylib.fuzzylib.modell.PdFuzzyRegelbasis;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.umfelddaten.PdUfdmsParameterFahrBahnZustandFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.umfelddaten.PdUfdmsParameterGlaetteFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.umfelddaten.PdUfdmsParameterSichtVerhaeltnisseFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.umfelddaten.PdUfdsParameterHelligkeitFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.umfelddaten.PdUfdsParameterNiederschlagsIntensitaetFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.umfelddaten.PdUfdsParameterSichtWeiteFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.umfelddaten.PdUfdsParameterWindGeschwindigkeitMittelWertFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.umfelddaten.PdUfdsParameterWindRichtungFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.verkehr.PdAnteilLkwFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.verkehr.PdBemessungsDichteFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.verkehr.PdGeschwindigkeitFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.verkehr.PdVerkehrsStaerkeStundeFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.verkehr.PdVerkehrsUnruheFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.verkehr.PdVerkehrsZustandFuzzy;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.verkehr.PdVerkehrsZustandUnruheFuzzy;
import de.bsvrz.sys.funclib.bitctrl.daf.BetriebsmeldungDaten;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.bitctrl.modell.Datensatz;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;
import de.bsvrz.sys.funclib.bitctrl.modell.Datum;
import de.bsvrz.sys.funclib.bitctrl.modell.ParameterDatensatz;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UfdsHelligkeit;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UfdsNiederschlagsIntensitaet;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UfdsSichtWeite;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UfdsWindGeschwindigkeitMittelWert;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UfdsWindRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UmfeldDatenMessStelle;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.StrassenTeilSegment;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Verwaltet die Parameter der Applikation. Meldet sich für die Fuzzy-Sets und
 * Regelbasen der Systemobjekte als Empf&auml;nger an.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class ParameterVerwaltung implements DatensatzUpdateListener {

	/** Logger. */
	private final Debug log = Debug.getLogger();

	/**
	 * Meldet sich für die Fuzzy-Sets und Regelbasen der Systemobjekte als
	 * Empfänger an.
	 *
	 * @param objekte
	 *            Menge von Systemobjekten
	 */
	ParameterVerwaltung(final Collection<SystemObjekt> objekte) {
		int anzahlObjekte, anzahlParameter;

		LogTools.log(log, SubSegAnaMsg.InfoStarteParameterVerwaltung);

		// Auf Parameter anmelden
		anzahlObjekte = 0;
		anzahlParameter = 0;
		for (final SystemObjekt so : objekte) {
			if (so instanceof MessQuerschnittAllgemein) {
				anzahlObjekte++;
				anzahlParameter += anmeldenEmpfaenger(so,
						PdAnteilLkwFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdBemessungsDichteFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdGeschwindigkeitFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdVerkehrsStaerkeStundeFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdVerkehrsUnruheFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdVerkehrsZustandFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdVerkehrsZustandUnruheFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdFuzzyRegelbasis.class);
			} else if (so instanceof StrassenTeilSegment) {
				final StrassenTeilSegment sts = (StrassenTeilSegment) so;
				if (sts.getMessQuerschnitte().isEmpty()) {
					// STS ohne MQ können ignoriert werden
					continue;
				}

				anzahlObjekte++;
				anzahlParameter += anmeldenEmpfaenger(so,
						PdAnteilLkwFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdBemessungsDichteFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdGeschwindigkeitFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdVerkehrsStaerkeStundeFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdVerkehrsUnruheFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdVerkehrsZustandFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdVerkehrsZustandUnruheFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdFuzzyRegelbasis.class);
			} else if (so instanceof UfdsHelligkeit) {
				anzahlObjekte++;
				anzahlParameter += anmeldenEmpfaenger(so,
						PdUfdsParameterHelligkeitFuzzy.class);
			} else if (so instanceof UfdsNiederschlagsIntensitaet) {
				anzahlObjekte++;
				anzahlParameter += anmeldenEmpfaenger(so,
						PdUfdsParameterNiederschlagsIntensitaetFuzzy.class);
			} else if (so instanceof UfdsWindGeschwindigkeitMittelWert) {
				anzahlObjekte++;
				anzahlParameter += anmeldenEmpfaenger(so,
						PdUfdsParameterWindGeschwindigkeitMittelWertFuzzy.class);
			} else if (so instanceof UfdsSichtWeite) {
				anzahlObjekte++;
				anzahlParameter += anmeldenEmpfaenger(so,
						PdUfdsParameterSichtWeiteFuzzy.class);
			} else if (so instanceof UfdsWindRichtung) {
				anzahlObjekte++;
				anzahlParameter += anmeldenEmpfaenger(so,
						PdUfdsParameterWindRichtungFuzzy.class);
			} else if (so instanceof UmfeldDatenMessStelle) {
				anzahlObjekte++;
				anzahlParameter += anmeldenEmpfaenger(so,
						PdUfdmsParameterFahrBahnZustandFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdUfdmsParameterGlaetteFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdUfdmsParameterSichtVerhaeltnisseFuzzy.class);
				anzahlParameter += anmeldenEmpfaenger(so,
						PdFuzzyRegelbasis.class);
			} else {
				LogTools.log(log,
						new BetriebsmeldungDaten(so.getSystemObject()),
						SubSegAnaMsg.FinerIgnoriereUnbekanntesSystemObject, so);
			}
		}

		LogTools.log(log, SubSegAnaMsg.ConfigParameterVerwaltungBereit,
				anzahlObjekte, anzahlParameter);
	}

	/**
	 * Hier passiert nichts, da nur die Autoupdatefunktion der Parameter
	 * aktiviert wurde und die Parameterwerte bei Bedarf ausgelesen werden.
	 */
	@Override
	public void datensatzAktualisiert(final DatensatzUpdateEvent event) {
		final SystemObjekt so = event.getObjekt();
		final Datum datum = event.getDatum();
		if (!datum.isValid()) {
			final Datensatz<?> datensatz = event.getDatensatz();
			LogTools.log(log, new BetriebsmeldungDaten(so.getSystemObject()),
					SubSegAnaMsg.ErrorUngueltigenParameterEmpfangen, so,
					datensatz.getAttributGruppe());
		} else {
			LogTools.log(log, new BetriebsmeldungDaten(so.getSystemObject()),
					SubSegAnaMsg.FineEmpfangeFuzzySetParameter, so, datum);
		}
	}

	/**
	 * Meldet sich als Empfänger eines Parameters des Systemobjekts an.
	 *
	 * @param so
	 *            ein Systemobjekt.
	 * @param parameter
	 *            der Parameter der empfangen werden soll.
	 * @return die Anzahl der angemeldeten Parameter (immer 1).
	 */
	private int anmeldenEmpfaenger(final SystemObjekt so,
			final Class<? extends ParameterDatensatz<?>> parameter) {
		ParameterDatensatz<?> pd;

		pd = so.getParameterDatensatz(parameter);
		LogTools.log(log, new BetriebsmeldungDaten(so.getSystemObject()),
				SubSegAnaMsg.FinerAnmeldenEmpfaengerParameter, so,
				pd.getAttributGruppe());
		pd.abrufenDatum();
		pd.addUpdateListener(this);

		return 1;
	}
}
